from cologer.main import Cologer
from cologer.field import Fore, Back, Style


loger = Cologer()
loger.set_field_fore(time=Fore.CYAN)
# coloring debug
loger.debug.fields.level.set_fore(Fore.BLACK).set_back(Back.MAGENTA)
# coloring info
loger.info.fields.level.set_fore(Fore.BLACK).set_back(Back.BLUE)
# coloring success
loger.success.fields.level.set_fore(Fore.BLACK).set_back(Back.GREEN)
# coloring warning
loger.warning.fields.level.set_fore(Fore.BLACK).set_back(Back.YELLOW)
# coloring error
loger.error.fields.level.set_fore(Fore.BLACK).set_back(Back.RED)


import re
import typing
import colorama
from cologer.lib import get_time, get_lineno, get_filename

colorama.init(autoreset=True)
Fore = colorama.Fore
Back = colorama.Back
Style = colorama.Style

pattern = re.compile(r'(?<=\{)[^}]*(?=\})+')


class Field:
    def __init__(self, name: str, level: str) -> None:
        self.name = name
        self.lv = level
        self._default = self._default_adapter()
        self.fore = ''
        self.back = ''
        self.style = ''

    def set_fore(self, fore: Fore):
        self.fore = fore
        return self

    def set_back(self, back: Back):
        self.back = back
        return self

    def set_style(self, style: Style):
        self.style = style
        return self

    @property
    def default(self):
        if callable(self._default):
            return self._default()
        return self._default

    def set_default(self, default: typing.Any):
        self._default = default
        return self

    def _default_adapter(self):
        name = self.name.lower()
        if name == 'time':
            return get_time
        elif name == 'level':
            return self.lv
        elif name == 'filename':
            return get_filename
        elif name == 'lineno':
            return get_lineno
        else:
            return ''

    def __str__(self) -> str:
        return self.fore + self.back + self.style + '{' + self.name + '}' + Style.RESET_ALL


class Fields:
    def __init__(self, fmt: str, level: str) -> None:
        self.fmt = fmt
        self.lv = level
        self.field_names = []
        self._set_field(fmt)

    def _set_field(self, fmt: str):
        for f_n in pattern.findall(fmt):
            self.field_names.append(f_n)
            setattr(self, f_n, Field(f_n, self.lv))

    def _get_final_str(self, *args, **kwargs):
        m = {}
        for f_n in self.field_names:
            field: Field = getattr(self, f_n)
            if f_n == 'message':
                u = ' '.join([str(u) for u in args])
                value = kwargs.get(f_n, u or field.default)
            else:
                value = kwargs.get(f_n, field.default)
            m[f_n] = str(field).format(**{f_n: value})
        return self.fmt.format(**m)



import sys

from cologer.field import Fields


class Level:
    def __init__(self, name: str, fmt: str) -> None:
        self.name = name.upper()
        self.visible = True
        self.fields = Fields(fmt, self.name)

    def invisible(self):
        self.visible = False

    def __call__(self, *args, **kwargs):
        if self.visible:
            sys.stdout.write(self.fields._get_final_str(*args, **kwargs)+'\n')



import os
import sys
from datetime import datetime


def get_time(): return datetime.now().strftime("%Y-%m-%d %H:%M:%S")


def get_cur_info():
    try:
        raise Exception
    except Exception:
        trace = sys.exc_info()[2].tb_frame
        while trace.f_back:
            trace = trace.f_back
        return (trace.f_code.co_filename, trace.f_lineno)


def get_filename(): return 'file:{}'.format(get_cur_info()[0])


def get_lineno(): return 'lineno:{}'.format(get_cur_info()[1])



from cologer.level import Level


class Cologer:
    """
    formatter:
        {time}      当前时间
        {filename}  文件名
        {lineno}    行号
    """
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_inst'):
            cls._inst = super().__new__(cls)
        return cls._inst

    def __init__(self) -> None:
        self._fmt = '{time} {level}: {message}'
        self.debug = Level('debug', self._fmt)
        self.info = Level('info', self._fmt)
        self.success = Level('success', self._fmt)
        self.warning = Level('warning', self._fmt)
        self.error = Level('error', self._fmt)

    def set_format(self, fmt: str):
        self._fmt = fmt
        for l, L in self.__dict__.items():
            if isinstance(L, Level):
                setattr(self, l, Level(l, self._fmt))

    def add_level(self, name: str):
        lv = Level(name, self._fmt)
        setattr(self, name, lv)
        return lv

    def set_field_fore(self, **kwargs):
        for l in self.__dict__.values():
            if isinstance(l, Level):
                for k, v in kwargs.items():
                    getattr(l.fields, k).set_fore(v)

    def set_field_back(self, **kwargs):
        for l in self.__dict__.values():
            if isinstance(l, Level):
                for k, v in kwargs.items():
                    getattr(l.fields, k).set_back(v)

    def set_field_style(self, **kwargs):
        for l in self.__dict__.values():
            if isinstance(l, Level):
                for k, v in kwargs.items():
                    getattr(l.fields, k).set_style(v)

    def set_field_default(self, **kwargs):
        for l in self.__dict__.values():
            if isinstance(l, Level):
                for k, v in kwargs.items():
                    getattr(l.fields, k).set_default(v)
